pro EB_LED_stack, stack_id, colors
;
;   creats a LED-stack like display in the widget plot window "stack_id"
;
;   determines from the widget which dimension is larger, and if the
;   number of elements is > 1, partitions that dimension to display
;   multiple LEDs
;
;   DPC   20070813
;	      20071029	added UVALUE to hold number of led segments
;		  20071030  added device, loadct calls
;
color_names = ["black", "white", "red", "orange", "yellow", "green", $
				"blue", "violet", "gray", $
				"skyblue", "grey68", "lawngreen", "pink", "plum", $
				"slateblue", "khaki light", "khaki dark", "goldenrod light",$
				"goldenrod dark", "rosybrown light", "rosybrown dark", $
				"palegreen light", "palegreen dark"]
;
;color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
color_values= [10,11,12,13,14,15,16,17,18,19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32]
device, decomposed=0
loadct, 13, /silent
;tvlct,        [0,        255,     255,     255,    255,        50,      0,       148,    190], $
;              [0,        255,       0,     165,    255,       205,      0,         0,    190], $
;              [0,        255,       0,       0,      0,        50,    255,       211,    190], 10
;
tclr = 			[[0, 	0, 	0], $ 	; black
				[255,255,255], $ 	; white
				[255,0,	0], $ 		; red
				[255,165,0], $ 		; orange
			   	[255,255,0], $ 		; yellow
          	   	[0,	255,0], $ 		; green
          	   	[0,	0,	255], $ 	; blue
          	   	[148,0,  211], $ 	; violet
          	   	[220,220,220], $ 	; gray
          	   	[135,206,235], $ 	; skyblue
          	   	[173,173,173], $ 	; grey68
          	   	[124,252,0], $ 		; lawngreen
          	   	[255,20, 147], $ 	; pink
          	   	[221,160,221], $ 	; plum
          	   	[106,90, 205], $ 	; slateblue
          	   	[255,246,143], $ 	; khaki light
          	   	[139,134, 78], $ 	; khaki dark
          	   	[255,193, 37], $ 	; goldenrod light
          	   	[139,105, 20], $ 	; goldenrod dark
          	   	[255,193,193], $ 	; rosybrown light
          	   	[139,105,105], $ 	; rosybrown dark
          	   	[154,255,154], $ 	; palegreen light
          	   	[84,139, 84	]]		; palegreen dark
;
tvlct, transpose(tclr), 10
;
nLED = N_ELEMENTS(colors)
;
;   get widget drawing size
;
geo_struct = WIDGET_INFO(stack_ID, /GEOMETRY)
;
;help, geo_struct, /struct
;
nx = geo_struct.scr_xsize
ny = geo_struct.scr_ysize
;
big_dimension = nx > ny
small_dimension = nx < ny
;
nx = big_dimension
ny = small_dimension
;
if(nLED gt 1) then begin
   ;
   ; partition x as nLED zones, with black in between
   ;
   xsize = FLOOR((nx-nLED+1) / nLED)
   ;
   ; compute buffer at start, end
   ;
   if(nx ne (xsize*nLED + nLED-1)) then begin
     ;
     ;    compute number of buffer pix on either end
     ;
     nbuffer = FIX(nx - (xsize*nLED + nLED-1))
     ;
     nb2 = FLOOR(nbuffer / 2)
     if(2*nb2 ne nbuffer) then begin
      nb1 = nb2+1
     endif else begin
      nb1 = nb2
     endelse
   endif else begin
     nb1 = 0
     nb2 = 0
   endelse
   ;
   ysize = small_dimension
   ;
   displ = bytarr(nx, ny)
   ;
   ix = 0
   for izone = 0, nLED-1 do begin
     ;
     ;    if there is a buffer zone to include at start, do so
     ;
     if(izone eq 0) then begin
      xstart = ix
      xend = xstart + nb1-1
      for jx = xstart, xend do begin
          for jy = 0, ysize-1 do begin
             displ[jx, jy] = byte(10)
          endfor
      endfor
      ix = xend + 1
     endif
     ;
     xstart = ix
     xend = ix + xsize-1
     for jx = xstart, xend do begin
      for jy = 0, ysize-1 do begin
          c_ind = WHERE(colors[izone] eq color_names, count)
          if(count ne 0) then begin
             displ[jx, jy] = byte(color_values[c_ind])
          endif else begin
             print, "ERROR - invalid color passed to LED stack. Color = ",colors[izone]
             displ[jx, jy] = byte(10)
          endelse
      endfor
     endfor
     ix = xend+1
     ;
     ;    add 1 pixel partition, so long as nb2 gt 0
     ;
     if(izone ne (nLED-1)) then begin
         jx = ix
         for jy = 0, ysize-1 do begin
          displ[jx, jy] = byte(10)
         endfor
         ix++
     endif
     ;
     if(izone eq (nLED-1)) then begin
      xstart = ix
      xend = xstart + nb2-1
      if(xend ge xstart) then begin
          for jx = xstart, xend do begin
              for jy = 0, ysize-1 do begin
                 displ[jx, jy] = byte(10)
              endfor
          endfor
          ix = xend + 1
      endif
     endif
   endfor
   ;
   ;  code for y>x stack
   ;
   if(geo_struct.scr_xsize ne nx) then begin
        ;
        ;  transpose displ
        ;
        displ = TRANSPOSE(displ)
    endif
;
endif   else begin
    ;
    ;  only one pixel
    ;
    c_one = colors[0]
    displ = bytarr(nx,ny)
    c_ind = WHERE(color_names eq c_one,count)
    if(count gt 0) then begin
       value = byte(color_values[c_ind])
    endif else begin
       print,"ERROR - invalid color value. Color = ",colors
       value = byte(10)
    endelse
    ;
    for ix = 0, nx-1 do begin
       for iy = 0, ny-1 do begin
         displ[ix, iy] = value
       endfor
    endfor
endelse
;
;   get window number
;
WIDGET_CONTROL, stack_id, GET_VALUE=win_no
;
;	save the number of segments in the wid
;
WIDGET_CONTROL, stack_id, SET_UVALUE=nLED
;
wset, win_no
;erase
;
;print, "calling tv for wid, window_no = ",stack_id,win_no," with colors = ",colors
;
tv, displ
;
end