pro E_B_Mode_Check
;
;	analyzes a single GPIPS POLCAT file to search for excess E and B mode signals
;	(which likely identify GPIPS fields with data reduction problems).
;
;	DPC	20160815
;
;--------------------------------
;
iplot = 0			; 0 = window, 1 = ps; 2 = none
jplot = 5			; 1 = PA vs PA, 2 = Offset angle histo, 3 = folded Offset histo, 4 = P vs radius; 5 = none
many_files = 1		; 0 = 1 file; 1 = directory of files
;
sPA_min = 15.0		; degrees
;
pa_start = -180.0	; bins
pa_end = 360.0
dpa_step = 10.0
npa = FLOOR((pa_end - pa_start)/dpa_step)+1.0
bin_xpa = (findgen(npa)+0.5)*dpa_step + pa_start	; bin center
bin_ypa = intarr(npa)
;
;	single file
;
if(many_files eq 0) then begin
	infile = dialog_pickfile(Title='Select one GPIPS POLCAT file')
	files = strarr(1)
	files[0] = infile
	nfiles = 1
endif
;
;	lots of files
;
if(many_files eq 1) then begin
	indir = dialog_pickfile(Title='Select GPIPS POLCAT directory',/dir)
	;
	;	for all entries in this directory
	;
	files = FILE_SEARCH(indir,'*.dat')
	nfiles = N_ELEMENTS(files)
	print, 'Found ',nfiles,' POLCAT files to test'
endif

;
pt_ratio_array = fltarr(nfiles)
;
for ifile = 0, nfiles-1 do begin
	;
	bin_ypa[*] = 0
	;
	infile = files[ifile]
	field_name = FILE_BASENAME(infile,'_POLCAT.dat')
	;
	;	read the file
	;
	data = read_polcat(infile)
	;
	;	for every entry, compute the field-centered position angle to the star
	;	assuming the center is at 50% of the image size in both directions
	;
	xcen = 511.5
	ycen = 512.5
	;
	nstars = N_ELEMENTS(data.s.num)
	;
	dx = data.s[*].x_det - xcen
	dy = data.s[*].y_det - ycen
	;
	field_pa = (720.0 -90.0 + 57.295 * atan(dy,dx)) mod 360.0 ; to go from xy -> PA
	field_rad = sqrt(dx*dx + dy*dy)
	;
	ind = where(data.s[*].sPA le sPA_min,ngood)
	;
	print, field_name,' has Ngood, with sPA <= ',sPA_min,' is ',ngood,' out of ',nstars,' stars in the file'
	;
	;	select polarization percentages, also
	;
	field_rad = field_rad[ind]
	field_pol = data.s[ind].P
	field_sPol = data.s[ind].sP
	;
	;	plot
	;
	device, decomposed =0
	loadct, 13
	tvlct, [0,255,255,0,0,148,235], [0,255,0,0,128,0,185], [0,255,0,255,0,211,15], 0
	black = 0
	white = 1
	red = 2
	blue = 3
	green = 4
	violet = 5
	yellow = 6
	low_color = 25
	dcolor = 55
	reg_color = [violet, green, blue, red, yellow]
	;
	if(iplot eq 0) then begin
		SET_PLOT, 'WIN'
		Window, 20, xsize=700, ysize=700, title=field_name
	endif
	;
	if(iplot eq 1) then begin
	;
		black = 1
		white = 0
		SET_PLOT, 'PS'
		!P.FONT=1
		DEVICE, /TIMES, /COLOR, FONT_SIZE=13, /PORTRAIT, /ENCAPSUL, $
			;YOFFSET=24.5,
			XSIZE=24,YSIZE=22
		;
		DEVICE, FILENAME=dialog_pickfile(Title='PS file to save')
	endif
	;
	!P.MULTI=[0]
	;
	if(jplot eq 1) then begin
		plot, [0, 360], [0, 360],xr=[0,360],xs=1,yr=[0,360],ys=1,/nodata,$
			xtitle='Field PA [deg]',ytitle='Polarization PA [deg]',$
			thick=2,color=white,charsize=1.5,xtickinterval=30,ytickinterval=30
		;
		oplot, field_pa[ind],data.s[ind].PA_deg, psym=4,color=blue,thick=4
		oplot, field_pa[ind],180.0+data.s[ind].PA_deg, psym=4,color=green,thick=4
		;
		xyouts, 45, 330, field_name, charsize=3.0, charthick=4.0, color=red
	endif
	;
	;	bin by diagonal zone
	;
	ibin = floor((field_pa[ind] - data.s[ind].PA_deg - (dpa_step / 2.0) - pa_start) / dpa_step)
	ibin >= 0
	ibin <= (npa-1)
	;
	for ib = 0, npa-1 do begin
		indb = where(ibin eq ib,nb)
		bin_ypa[ib] = nb
	endfor
	;
	;erase
	;
	;	plot, analyze the histogram
	;
	average = moment(bin_ypa)
	peak_to_average = max(bin_ypa) / (average[0])
	;print, 'Peak to Average ratio = ',peak_to_average
	;
	;	do a 90 deg fold
	;
	fxbin = (720.0 + bin_xpa) mod 90.0
	nnewbins = ROUND(90.0 / dpa_step)
	newxbins = dpa_step * (findgen(nnewbins)+0.5)
	;
	newy = fltarr(nnewbins)
	for k = 0, nnewbins-1 do begin
		indn = WHERE(fxbin eq newxbins[k], nk)
		newy[k] = total(bin_ypa[indn])
	endfor
	;
	pk = max(newy)
	tr = min(newy)
	pt_ratio = float(pk - tr) / float(pk + tr)
	;
	;print, 'Peak-to-Trough ratio = ',pt_ratio
	;
	;	save in array
	;
	pt_ratio_array[ifile] = pt_ratio
	;
	ymax = max(bin_ypa) * 1.05
	;
	if(jplot eq 2) then begin
		plot, bin_xpa, bin_ypa, xr=[pa_start,pa_end],xs=1,yr = [0.,ymax],ys=1,$
			xtitle='Bin center offset [deg]',ytitle='Bin star count',$
			thick=2,color=white,charsize=1.5,xtickinterval=45,ytickinterval=10,/nodata
		;
		;	E, B zones
		;
		e0 = -180.0
		e1 = e0 + 22.5
		this = 'e'
		b0 = 0.0
		b1 = 0.0
		;
		WHILE (e0 lt 360.0 and b0 lt 360.0) do begin
			case this of
				'e':	begin
							xp = [e0, e1, e1, e0, e0]
							yp = [0, 0, ymax, ymax, 0]
							polyfill, xp, yp, color=blue, /line_fill, spacing=0.5, orientation=45, thick=2
							;
							this = 'b'
							b0 = e1
							b1 = (b0 + 45.0) < 360.0
						end
				'b':	begin
							xp = [b0, b1, b1, b0, b0]
							polyfill, xp, yp, color=green, /line_fill, spacing=0.5, orientation=-45, thick=2
							;
							this = 'e'
							e0 = b1
							e1 = (e0 + 45) < 360.0
						end
			endcase
		ENDWHILE
		plot, bin_xpa, bin_ypa, xr=[pa_start,pa_end],xs=1,yr = [0.,ymax],ys=1,/noerase,$
			xtitle='Bin center offset [deg]',ytitle='Bin star count',$
			thick=2,color=white,charsize=1.5,xtickinterval=45,ytickinterval=10,/nodata
		;
		oplot, bin_xpa, bin_ypa, psym=-4, thick=6, symsize=2, color=red
		;
		xyouts, bin_xpa[3], 0.93*ymax, field_name, charsize=3.0, charthick=4.0, color=red
	endif
	;
	if(jplot eq 3) then begin
		plot, newxbins, newy, xr=[0.,90.],xs=1,yr = [0.,1.05*max(newy)],ys=1,$
			xtitle='Folded Bin center offset [deg]',ytitle='Folded Bin star count',$
			thick=2,color=white,charsize=1.5,xtickinterval=45,ytickinterval=10,/nodata
		;
		oplot, newxbins, newy, psym=-4, thick=6, symsize=2, color=red
		;
		xyouts, newxbins[2], 0.93*max(newy), field_name, charsize=3.0, charthick=4.0, color=red
	endif
	;
	;	4th plot - P vs rad
	;
	if(jplot eq 4) then begin
		plot, field_rad, field_pol, xr=[0.,750.],xs=1,yr = [0.,8.],ys=1,$
			xtitle='Center radial offset [pix]',ytitle='Polarization Percentage',$
			thick=2,color=white,charsize=1.5,xtickinterval=100,ytickinterval=1.0,/nodata
		;
		oplot, field_rad, field_pol, psym=4, thick=6, symsize=0.5, color=red
		;
		xyouts, 50., 7.5, field_name, charsize=3.0, charthick=4.0, color=red
		;
		a = poly_fit(0.01*field_rad, field_pol,1, sigma=sigma_a,measure_errors=field_sPol)
		oplot, [0.,750.], a[0]+a[1]*[0.,7.500],psym=-3, thick=5, color=blue
		xyouts, 50, 7.0, string(a[0],sigma_a[0],a[1],sigma_a[1],$
			format='("P=",f7.3,"+/-",f6.3," + ",f7.4,"+/-",f6.4," Rad [100pix] + ")'),$
			charsize=2.0,color=white,charthick=4
;		xyouts, 100, 6.2, string(a[2],sigma_a[2],$
;			format='(f8.5,"+/-",f8.5," Rad^2 [100pix]")'),$
;			charsize=2.0,color=white,charthick=4
	endif
	;
	;
endfor
;
;	all files evaluated, save results
;
if(many_files eq 1) then begin
	outfile = dialog_pickfile(Title='File to save PT ratios into')
	openw, lu_out, outfile, /get_lun
	;
	for i = 0, nfiles-1 do begin
		printf, lu_out, i+1, file_basename(files[i],"_POLCAT.dat"),pt_ratio_array[i],$
			format='(i4.4,1x,a8,1x,f8.4)'
	endfor
	free_lun, lu_out
endif
;
if(iplot eq 1) then begin
	DEVICE,/CLOSE
	;
	SET_PLOT, 'WIN'
endif
if(iplot eq 0) then begin
	cursor, xx, yy, /wait
	wait, 0.3
	;
	WDELETE, 20
endif
;
end